IF( NOT ROBOTINODEV_DIR )
	IF( WIN32 )
		FATAL_ERROR("ROBOTINODEV_DIR not set")
		ELSE( WIN32 )
				SET( ROBOTINODEV_DIR "/opt/robotino" )
		ENDIF( WIN32 )
ENDIF( NOT ROBOTINODEV_DIR )
	
FIND_PATH(
   ROBOTINO_DEV_INCLUDE_DIR 
   NAMES
   rec/robotino/rpc/Client.h
   PATHS
   ${ROBOTINODEV_DIR}/include
   NO_DEFAULT_PATH
)

FIND_LIBRARY(
  REC_ROBOTINO_RPC_RELEASE_LIBRARY
  NAMES rec_robotino_rpc
  PATHS "${ROBOTINODEV_DIR}/lib"
)
SET( ROBOTINO_DEV_LIBRARIES REC_ROBOTINO_RPC_RELEASE_LIBRARY )

IF( WIN32 )
	FIND_LIBRARY(
		REC_ROBOTINO_RPC_DEBUG_LIBRARY
		NAMES rec_robotino_rpcd
		PATHS "${ROBOTINODEV_DIR}/lib"
	)

	FIND_FILE( REC_ROBOTINO_RPC_DEBUG_DLL rec_robotino_rpcd.dll ${ROBOTINODEV_DIR}/bin NO_DEFAULT_PATH )
	FIND_FILE( REC_ROBOTINO_RPC_RELEASE_DLL rec_robotino_rpc.dll ${ROBOTINODEV_DIR}/bin NO_DEFAULT_PATH )

	SET(
		ROBOTINO_DEV_RELEASE_DLLS
		"${REC_ROBOTINO_RPC_RELEASE_DLL}"
	)

	SET(
		ROBOTINO_DEV_DEBUG_DLLS
		"${REC_ROBOTINO_RPC_DEBUG_DLL}"
	)
	
	FILE( GLOB REC_ROBOTINO_DEV_PDBS "${ROBOTINODEV_DIR}/bin/*.pdb" )
ENDIF( WIN32 )
	
SET( ROBOTINO_DEV_FOUND 0 )

IF( ROBOTINO_DEV_INCLUDE_DIR )
	SET( ROBOTINO_DEV_FOUND 1 )
ELSE( ROBOTINO_DEV_INCLUDE_DIR )
	IF( ROBOTINO_DEV_FIND_REQUIRED )
		MESSAGE(FATAL_ERROR "Could not find ROBOTINO_DEV. Download from http://wiki.openrobotino.org")
	ENDIF( ROBOTINO_DEV_FIND_REQUIRED )
ENDIF( ROBOTINO_DEV_INCLUDE_DIR )

IF( ROBOTINO_DEV_FOUND )
	INCLUDE_DIRECTORIES(
		${ROBOTINO_DEV_INCLUDE_DIR}
	)
	
	IF( WIN32 )
		FOREACH( var ${ROBOTINO_DEV_LIBRARIES} )
			STRING( REPLACE "_RELEASE_" "_" LIB_NAME "${var}" )
			STRING( REPLACE "_RELEASE_" "_DEBUG_" DEBUG_LIB_NAME "${var}" )
			SET( ${LIB_NAME} optimized ${${var}} debug ${${DEBUG_LIB_NAME}} )
		ENDFOREACH( var )
	ELSE( WIN32 )
		FOREACH( var ${ROBOTINO_DEV_LIBRARIES} )
			STRING( REPLACE "_RELEASE_" "_" LIB_NAME "${var}" )
			SET( ${LIB_NAME} ${${var}} )
		ENDFOREACH( var )
	ENDIF( WIN32 )
	
ENDIF( ROBOTINO_DEV_FOUND )

MACRO( COPY_ROBOTINO_DEV_DLLS releaseTarget debugTarget )
  FOREACH( var ${ROBOTINO_DEV_RELEASE_DLLS} )
    CONFIGURE_FILE( ${var} ${releaseTarget} COPYONLY )
  ENDFOREACH( var )
  FOREACH( var ${ROBOTINO_DEV_DEBUG_DLLS} )
    CONFIGURE_FILE( ${var} ${debugTarget} COPYONLY )
  ENDFOREACH( var )
ENDMACRO( COPY_ROBOTINO_DEV_DLLS )

MACRO( COPY_ROBOTINO_DEV_PDBS releaseTarget debugTarget )
  FOREACH( var ${REC_ROBOTINO_DEV_PDBS} )
    CONFIGURE_FILE( ${var} ${releaseTarget} COPYONLY )
  ENDFOREACH( var )
ENDMACRO()

